-- Communicating Haskell Processes.
-- Copyright (c) 2008--2010, University of Kent.
-- All rights reserved.
-- 
-- Redistribution and use in source and binary forms, with or without
-- modification, are permitted provided that the following conditions are
-- met:
--
--  * Redistributions of source code must retain the above copyright
--    notice, this list of conditions and the following disclaimer.
--  * Redistributions in binary form must reproduce the above copyright
--    notice, this list of conditions and the following disclaimer in the
--    documentation and/or other materials provided with the distribution.
--  * Neither the name of the University of Kent nor the names of its
--    contributors may be used to endorse or promote products derived from
--    this software without specific prior written permission.
--
-- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
-- IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
-- THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
-- PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
-- CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
-- EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
-- PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
-- PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
-- LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-- NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
-- SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

-- | Various (orphan) instances for monads from the chp and chp-plus libraries
-- for classes from the mtl library.
--
-- There are 'MonadIO' instances for:
--
-- * 'CHP'
-- 
-- * 'CHPTest'
-- 
-- * 'Composed'
--
-- There are 'MonadCHP' instances of the form @instance MonadCHP m => MonadCHP
-- (t m)@, where t is one of the monad transformers:
--
-- * WriterT (lazy and strict)
--
-- * StateT (lazy and strict)
--
-- * RWST (lazy and strict)
--
-- * 'ReaderT'
--
-- * 'ErrorT'
--
-- * 'ContT'
module Control.Concurrent.CHP.MTL where

import Control.Concurrent.CHP
import Control.Concurrent.CHP.Composed
import Control.Concurrent.CHP.Test
import Control.Monad.Cont
import Control.Monad.Error
import Control.Monad.Reader
import qualified Control.Monad.RWS.Lazy as LR
import qualified Control.Monad.RWS.Strict as SR
import qualified Control.Monad.State.Lazy as LS
import qualified Control.Monad.State.Strict as SS
import qualified Control.Monad.Writer.Lazy as LW
import qualified Control.Monad.Writer.Strict as SW
import Control.Monad.Trans
import Data.Monoid

instance MonadIO CHP where
  liftIO = liftIO_CHP

instance (Monoid w, MonadCHP m) => MonadCHP (LW.WriterT w m) where
  liftCHP = lift . liftCHP

instance (Monoid w, MonadCHP m) => MonadCHP (SW.WriterT w m) where
  liftCHP = lift . liftCHP

instance MonadCHP m => MonadCHP (LS.StateT s m) where
  liftCHP = lift . liftCHP

instance MonadCHP m => MonadCHP (SS.StateT s m) where
  liftCHP = lift . liftCHP

instance (Monoid w, MonadCHP m) => MonadCHP (LR.RWST r w s m) where
  liftCHP = lift . liftCHP

instance (Monoid w, MonadCHP m) => MonadCHP (SR.RWST r w s m) where
  liftCHP = lift . liftCHP

instance (Error e, MonadCHP m) => MonadCHP (ErrorT e m) where
  liftCHP = lift . liftCHP

instance MonadCHP m => MonadCHP (ReaderT r m) where
  liftCHP = lift . liftCHP

instance MonadCHP m => MonadCHP (ContT r m) where
  liftCHP = lift . liftCHP

instance MonadIO CHPTest where
  liftIO = liftCHP . liftIO

instance MonadIO Composed where
  liftIO = liftCHP . liftIO


